#!/usr/bin/env bash
# kate:hl bash;

#
# 2  days of minute
# 14 days of 15 minute  avg
# 31 days of hourly     avg, min, max
# 1  year of daily      avg, min, max
#
# Celsius
MIN=-55
MAX=125
# Fahrenheit
MIN=-67
MAX=257
rrdtool create temperatures.rrd \
         --step 60 \
         DS:s1:GAUGE:120:$MIN:$MAX \
         DS:s2:GAUGE:120:$MIN:$MAX \
         DS:s3:GAUGE:120:$MIN:$MAX \
         DS:s4:GAUGE:120:$MIN:$MAX \
         DS:s5:GAUGE:120:$MIN:$MAX \
         DS:s6:GAUGE:120:$MIN:$MAX \
         DS:s7:GAUGE:120:$MIN:$MAX \
         DS:s8:GAUGE:120:$MIN:$MAX \
         DS:s9:GAUGE:120:$MIN:$MAX \
         DS:s10:GAUGE:120:$MIN:$MAX \
         DS:s11:GAUGE:120:$MIN:$MAX \
         DS:s12:GAUGE:120:$MIN:$MAX \
         DS:s13:GAUGE:120:$MIN:$MAX \
         DS:s14:GAUGE:120:$MIN:$MAX \
         DS:s15:GAUGE:120:$MIN:$MAX \
         DS:s16:GAUGE:120:$MIN:$MAX \
         RRA:AVERAGE:0.5:1:2880 \
         RRA:AVERAGE:0.5:15:1344 \
         RRA:AVERAGE:0.5:60:744 \
         RRA:MIN:0.5:60:744 \
         RRA:MAX:0.5:60:744 \
         RRA:AVERAGE:0.5:1440:365 \
         RRA:MIN:0.5:1440:365 \
         RRA:MAX:0.5:1440:365
